"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getParamNames_1 = require("./getParamNames");
exports.formatToSchema = (schema, data) => {
    if (invalidSchema(schema) || !data) {
        return;
    }
    if (schema.ObjectToFormatFromData) {
        data = data[schema.ObjectToFormatFromData];
    }
    if (checkIfSchemaIsList(schema)) {
        return formatAsList(schema.format, data);
    }
    else {
        return formatAsObj(schema.format, data);
    }
};
/*
 * Mapper to format Json to Object schema.
*/
const formatAsObj = (schema, data) => {
    if (Array.isArray(data)) {
        data = data[0]; // Taking 1st object in array if data passed is array and schema is not list.
    }
    const keys = Object.keys(schema);
    return formatter(keys, schema, data);
};
/*
 * Mapper to format Json to List schema.
*/
const formatAsList = (schema, data) => {
    if (!Array.isArray(data)) {
        data = [data];
    }
    const keys = Object.keys(schema);
    return data.map((item) => {
        return formatter(keys, schema, item);
    });
};
/*
 * Processor Function which returns single transformed object.
*/
const formatter = (keys, schema, data) => {
    const regex = /\$/g;
    let obj = {};
    keys.forEach((key) => {
        let dataField = schema[key].dataField;
        if (schema[key].type && schema[key].type.toUpperCase() === 'FUNCTION') {
            dataField = new Function(`return ${dataField}`)();
            obj = Object.assign(Object.assign({}, obj), { [key]: transformFn(dataField, getParamsForTransformFn(dataField, data)) });
        }
        else {
            if (Array.isArray(dataField)) {
                const item = JSON.stringify(dataField);
                const mapFields = JSON.parse(item.replace(/\$/g, ""));
                obj = Object.assign(Object.assign({}, obj), { [key]: mapMultipleFields(mapFields, data) });
            }
            else {
                const mapField = dataField.replace(regex, "");
                obj = Object.assign(Object.assign({}, obj), { [key]: data[mapField] });
            }
        }
    });
    return obj;
};
const invalidSchema = (schema) => {
    return !schema || !schema.format;
};
const checkIfSchemaIsList = (schema) => {
    return schema.isList;
};
const mapMultipleFields = (fields, item) => {
    const dataset = [];
    fields.forEach((ele) => {
        if (ele instanceof Function) {
            dataset.push(transformFn(ele, getParamsForTransformFn(ele, item)));
        }
        else {
            dataset.push(item[ele]);
        }
    });
    return dataset;
};
const getParamsForTransformFn = (fn, data) => {
    return getParamNames_1.getParameterNames(fn).map((param) => {
        return data[param];
    });
};
const transformFn = (fn, params) => {
    return fn.apply(fn, params);
};
